/**
 * sigma_formatter.js
 *
 * Specialized formatters for Sigma rule data
 */
const { wrapText, formatDate, formatNumber } = require('./text_formatter');
const { formatKeyValueTable, formatTable } = require('./table_formatter');
const colors = require('../utils/colors');

/**
 * Format Sigma rule details for CLI display
 * 
 * @param {Object} ruleDetails - The rule details object
 * @returns {Object} Formatted rule details
 */
function formatSigmaDetails(ruleDetails) {
  if (!ruleDetails) {
    return null;
  }
  
  // Create a flattened object for display in CLI table format
  return {
    'ID': ruleDetails.id || 'Unknown',
    'Title': ruleDetails.title || 'Untitled Rule',
    'Description': ruleDetails.description || 'No description provided',
    'Author': ruleDetails.author || 'Unknown author',
    'Severity': ruleDetails.severity || 'Unknown',
    'Status': ruleDetails.status || 'Unknown',
    'Created': formatDate(ruleDetails.date),
    'Modified': formatDate(ruleDetails.modified),
    'Detection': ruleDetails.detectionExplanation || 'No detection specified',
    'False Positives': Array.isArray(ruleDetails.falsePositives) ? 
                        ruleDetails.falsePositives.join(', ') : 'None specified',
    'Tags': Array.isArray(ruleDetails.tags) ? 
            ruleDetails.tags.join(', ') : 'None',
    'References': Array.isArray(ruleDetails.references) ? 
                  ruleDetails.references.join(', ') : 'None'
  };
}

/**
 * Format Sigma statistics for CLI display
 * 
 * @param {Object} stats - The statistics object
 * @returns {Object} Formatted stats
 */
function formatSigmaStats(stats) {
  if (!stats) {
    return { error: 'No statistics data available' };
  }
  
  // Create a simplified object suitable for table display
  const formattedStats = {
    'Last Update': formatDate(stats.lastUpdate),
    'Total Rules': formatNumber(stats.totalRules),
    'Database Health': `${stats.databaseHealth.contentPercentage}% Complete`,
    
    // OS breakdown
    'Windows Rules': formatNumber(stats.operatingSystems.windows),
    'Linux Rules': formatNumber(stats.operatingSystems.linux),
    'macOS Rules': formatNumber(stats.operatingSystems.macos),
    'Other OS Rules': formatNumber(stats.operatingSystems.other),
  };
  
  // Add severity levels
  if (stats.severityLevels && Array.isArray(stats.severityLevels)) {
    stats.severityLevels.forEach(level => {
      const levelName = level.level 
        ? level.level.charAt(0).toUpperCase() + level.level.slice(1) 
        : 'Unknown';
      
      formattedStats[`${levelName} Severity`] = formatNumber(level.count);
    });
  }
  
  // Add top MITRE tactics if available
  if (stats.mitreTactics && Array.isArray(stats.mitreTactics)) {
    stats.mitreTactics.slice(0, 5).forEach(tactic => { // Only top 5
      const formattedTactic = tactic.tactic
        .replace(/-/g, ' ')
        .split(' ')
        .map(word => word.charAt(0).toUpperCase() + word.slice(1))
        .join(' ');
          
      formattedStats[`MITRE: ${formattedTactic}`] = formatNumber(tactic.count);
    });
  }
    
  return formattedStats;
}

/**
 * Format Sigma search results for CLI display
 * 
 * @param {Object} searchResults - The search results object
 * @returns {Object} Formatted search results
 */
function formatSigmaSearchResults(searchResults) {
  if (!searchResults || !searchResults.results) {
    return { error: 'No search results available' };
  }
  
  // Map raw results to formatted results
  const formattedResults = searchResults.results.map(rule => {
    // Get logsource.product field
    let osProduct = 'N/A';
    
    if (rule.logsource && rule.logsource.product) {
      osProduct = rule.logsource.product;
    }
    
    return {
      id: rule.id || '',
      title: rule.title || '',
      author: rule.author || 'Unknown',
      level: rule.level || 'medium',
      osProduct: osProduct,
      tags: rule.tags || []
    };
  });
  
  return {
    results: formattedResults,
    totalCount: searchResults.totalCount || 0
  };
}

/**
 * Render formatted Sigma details as a string
 * 
 * @param {Object} formattedDetails - Formatted details object
 * @returns {string} CLI-ready output
 */
function renderSigmaDetails(formattedDetails) {
  if (!formattedDetails) {
    return 'No details available';
  }
  
  return formatKeyValueTable(formattedDetails, 24, 50).join('\n');
}

/**
 * Render formatted Sigma statistics as a string
 * 
 * @param {Object} formattedStats - Formatted statistics object
 * @returns {string} CLI-ready output
 */
function renderSigmaStats(formattedStats) {
  if (!formattedStats) {
    return 'No statistics available';
  }
  
  return formatKeyValueTable(formattedStats, 25, 30, colors.statsKey).join('\n');
}

/**
 * Render formatted Sigma search results as a string
 * 
 * @param {Object} formattedResults - Formatted search results object
 * @returns {string} CLI-ready output
 */
function renderSigmaSearchResults(formattedResults) {
  if (!formattedResults || formattedResults.error) {
    return formattedResults.error || 'No search results available';
  }
  
  const headers = ['#', 'Title', 'OS/Product', 'ID'];
  const widths = [5, 32, 15, 13];
  
  // Create rows from results
  const rows = formattedResults.results.map((rule, index) => {
    return [
      (index + 1).toString(),
      rule.title.substring(0, 31), // Trim to fit column width
      rule.osProduct.substring(0, 14),
      rule.id.substring(0, 12)
    ];
  });
  
  // Generate table
  const tableLines = formatTable(headers, rows, widths);
  
  // Add count footer
  const countLine = colors.count(`${formattedResults.totalCount} rows in set`);
  
  return [...tableLines, '', countLine].join('\n');
}

module.exports = {
  formatSigmaDetails,
  formatSigmaStats,
  formatSigmaSearchResults,
  renderSigmaDetails,
  renderSigmaStats,
  renderSigmaSearchResults
};