/**
 * text_formatter.js
 *
 * Common text formatting utilities for CLI output
 */

/**
 * Wraps text at specified length, handling different data types
 * 
 * @param {any} text - Text to wrap (will be converted to string)
 * @param {number} maxWidth - Maximum line width
 * @returns {string[]} Array of wrapped lines
 */
function wrapText(text, maxWidth = 80) {
    // Handle non-string or empty inputs
    if (text === undefined || text === null) return [''];
    
    // Convert to string and normalize newlines
    const normalizedText = String(text).replace(/\n/g, ' ');
    
    // If text fits in one line, return it as a single-element array
    if (normalizedText.length <= maxWidth) return [normalizedText];
    
    const words = normalizedText.split(' ');
    const lines = [];
    let currentLine = '';
    
    for (const word of words) {
      // Skip empty words (could happen if there were multiple spaces)
      if (!word) continue;
      
      // If adding this word would exceed max width
      if ((currentLine.length + word.length + (currentLine ? 1 : 0)) > maxWidth) {
        // Push current line if not empty
        if (currentLine) {
          lines.push(currentLine);
          currentLine = '';
        }
        
        // If the word itself is longer than maxWidth, we need to split it
        if (word.length > maxWidth) {
          let remaining = word;
          while (remaining.length > 0) {
            const chunk = remaining.substring(0, maxWidth);
            lines.push(chunk);
            remaining = remaining.substring(maxWidth);
          }
        } else {
          currentLine = word;
        }
      } else {
        // Add word to current line
        currentLine = currentLine ? `${currentLine} ${word}` : word;
      }
    }
    
    // Add the last line if not empty
    if (currentLine) {
      lines.push(currentLine);
    }
    
    return lines;
  }
  
  /**
   * Formats a date string for display
   * 
   * @param {string} dateString - Date string to format
   * @returns {string} Formatted date string
   */
  function formatDate(dateString) {
    if (!dateString) return 'Unknown';
    
    try {
      const date = new Date(dateString);
      return date.toLocaleString();
    } catch (error) {
      return dateString;
    }
  }
  
  /**
   * Formats a number with locale-specific thousands separators
   * 
   * @param {number} num - Number to format
   * @returns {string} Formatted number string
   */
  function formatNumber(num) {
    if (typeof num !== 'number') return String(num || '');
    return num.toLocaleString();
  }
  
  module.exports = {
    wrapText,
    formatDate,
    formatNumber
  };