/**
 * sigma_basic_search_handler.js
 * 
 * Handles basic keyword search commands for Sigma rules
 */
const { searchAndConvertRules } = require('../../services/sigma/sigma_basic_search_service');
const logger = require('../../utils/logger');
const { handleError } = require('../../utils/error_handler');
const { getSearchResultBlocks } = require('../../blocks/sigma/sigma_search_results_block');
const { getFileName } = require('../../utils/file_utils');
const { extractPaginationParams } = require('../../services/sigma/utils/search_pagination_utils');

const FILE_NAME = getFileName(__filename);
const MAX_RESULTS_THRESHOLD = 99;

/**
 * Handles the basic sigma-search command
 * @param {Object} command - Slack command object
 * @param {Function} respond - Function to send response
 */
const handleBasicSearch = async (command, respond) => {
  try {
    logger.debug(`${FILE_NAME}: Processing basic sigma-search command: ${JSON.stringify(command.text)}`);

    if (!command || !command.text) {
      logger.warn(`${FILE_NAME}: Empty command received for sigma-search`);
      await respond('Invalid command. Usage: /sigma-search [keyword]');
      return;
    }

    // Extract keyword and pagination parameters
    const { text: keyword, page, pageSize } = extractPaginationParams(command.text);

    if (!keyword) {
      logger.warn(`${FILE_NAME}: Missing keyword in sigma-search command`);
      await respond('Invalid command: missing keyword. Usage: /sigma-search [keyword]');
      return;
    }

    logger.info(`${FILE_NAME}: Searching for rules with keyword: ${keyword} (page ${page}, size ${pageSize})`);

    await respond({
      text: 'Searching for rules... This may take a moment.',
      response_type: 'ephemeral'
    });

    // Perform the search
    const searchResult = await searchAndConvertRules(keyword, page, pageSize);
    
    if (!searchResult.success) {
      logger.error(`${FILE_NAME}: Search failed: ${searchResult.message}`);
      await respond({
        text: `Search failed: ${searchResult.message}`,
        response_type: 'ephemeral'
      });
      return;
    }

    // Get total count for validation
    const totalCount = searchResult.pagination?.totalResults || 0;

    // Check if search returned too many results
    if (totalCount > MAX_RESULTS_THRESHOLD) {
      logger.warn(`${FILE_NAME}: Search for "${keyword}" returned too many results (${totalCount})`);
      searchResult.tooManyResults = true;
    }

    if (!searchResult.results || searchResult.results.length === 0) {
      if (totalCount > 0) {
        logger.warn(`${FILE_NAME}: No rules found on page ${page} for "${keyword}", but ${totalCount} total matches exist`);
        await respond({
          text: `No rules found on page ${page} for "${keyword}". Try a different page or refine your search.`,
          response_type: 'ephemeral'
        });
      } else {
        logger.warn(`${FILE_NAME}: No rules found matching "${keyword}"`);
        await respond({
          text: `No rules found matching "${keyword}"`,
          response_type: 'ephemeral'
        });
      }
      return;
    }

    const isCliRequest = command.channel_id === 'cli' || command.channel_name === 'cli';

    if (isCliRequest) {
      // For CLI, return raw data
      await respond({
        responseData: searchResult.results,
        response_type: 'cli'
      });
    } else {
      // For Slack, generate Block Kit blocks
      try {
        let blocks = getSearchResultBlocks(keyword, searchResult.results, searchResult.pagination);
        
        // Add warning for too many results
        if (searchResult.tooManyResults) {
          blocks.splice(1, 0, {
            "type": "section",
            "text": {
              "type": "mrkdwn",
              "text": `:warning: Your search for "${keyword}" returned ${totalCount} results, which is a lot. Displaying the first page. Consider using a more specific keyword for narrower results.`
            }
          });
        }
        
        // Determine visibility
        const isEphemeral = totalCount > 20;
        
        // Send response
        await respond({
          blocks: blocks,
          response_type: isEphemeral ? 'ephemeral' : 'in_channel'
        });
        
        logger.debug(`${FILE_NAME}: Response sent successfully`);
      } catch (blockError) {
        await handleError(blockError, `${FILE_NAME}: Block generation`, respond, {
          responseType: 'in_channel',
          customMessage: `Found ${searchResult.results.length} of ${totalCount} rules matching "${keyword}" (page ${page} of ${searchResult.pagination?.totalPages || 1}). Use /sigma-details [id] to view details.`
        });
      }
    }
  } catch (error) {
    await handleError(error, `${FILE_NAME}: Search command handler`, respond, {
      responseType: 'ephemeral'
    });
  }
};

module.exports = {
  handleBasicSearch
};