/**
 * sigma_search_entry_handler.js
 * 
 * Entry point for all sigma search commands
 * Determines whether to use basic or complex search
 */
const { handleBasicSearch } = require('./sigma_basic_search_handler');
const { handleComplexSearch } = require('./sigma_complex_search_handler');
const logger = require('../../utils/logger');
const { getFileName } = require('../../utils/file_utils');

const FILE_NAME = getFileName(__filename);

/**
 * Detects if a command is a complex search query
 * @param {Object} command - Slack command object
 * @returns {boolean} True if complex search
 */
function isComplexSearch(command) {
  if (!command || !command.text) return false;
  
  const text = command.text.trim().toLowerCase();
  
  // Check for complex query indicators
  return text.startsWith('where ') || 
         text.includes(' and ') || 
         text.includes(' or ') || 
         /title:/.test(text) || 
         /logsource\./.test(text) ||
         /tags:/.test(text) ||
         /contains\(/.test(text);
}

/**
 * Main entry point for sigma search commands
 * Routes to appropriate handler based on query type
 * @param {Object} command - Slack command object
 * @param {Function} respond - Function to send response
 */
const handleCommand = async (command, respond) => {
  try {
    logger.debug(`${FILE_NAME}: Processing sigma search command: ${command?.text || 'empty'}`);
    
    if (!command) {
      logger.error(`${FILE_NAME}: No command object provided`);
      await respond({
        text: 'Error processing command: Invalid command format',
        response_type: 'ephemeral'
      });
      return;
    }
    
    // Determine search type and route to appropriate handler
    if (isComplexSearch(command)) {
      logger.info(`${FILE_NAME}: Detected complex search query, routing to complex handler`);
      await handleComplexSearch(command, respond);
    } else {
      logger.info(`${FILE_NAME}: Detected basic keyword search, routing to basic handler`);
      await handleBasicSearch(command, respond);
    }
  } catch (error) {
    logger.error(`${FILE_NAME}: Error in search entry handler: ${error.message}`);
    await respond({
      text: `Error processing search command: ${error.message}`,
      response_type: 'ephemeral'
    });
  }
};

module.exports = {
  handleCommand,
  isComplexSearch
};