/**
 * sigma_basic_search_service.js
 * 
 * Service for basic keyword searches of Sigma rules
 */
const { searchRules } = require('../../sigma_db/queries');
const { convertSigmaRule } = require('./sigma_converter_service');
const logger = require('../../utils/logger');
const { getFileName } = require('../../utils/file_utils');
const { validateKeyword, validatePagination } = require('./utils/search_validation_utils');
const { createPaginationInfo, createEmptyResponse } = require('./utils/search_pagination_utils');

const FILE_NAME = getFileName(__filename);

/**
 * Searches for Sigma rules by keyword
 * @param {string} keyword - Keyword to search for
 * @param {number} page - Page number (1-based)
 * @param {number} pageSize - Results per page
 * @returns {Promise<Object>} Search results with pagination
 */
async function searchSigmaRules(keyword, page = 1, pageSize = 10) {
  // Validate keyword
  const keywordValidation = validateKeyword(keyword);
  if (!keywordValidation.isValid) {
    return {
      success: false,
      message: keywordValidation.message
    };
  }
  
  // Validate pagination
  const { page: validatedPage, pageSize: validatedPageSize, offset } = 
    validatePagination(page, pageSize);
  
  logger.info(`${FILE_NAME}: Searching for Sigma rules with keyword: "${keywordValidation.trimmedKeyword}" (page ${validatedPage}, size ${validatedPageSize})`);
  
  try {
    // Perform the database search
    const searchResult = await searchRules(keywordValidation.trimmedKeyword, validatedPageSize, offset);
    
    // Extract results and total count
    let allResults = [];
    let totalCount = 0;
    
    if (searchResult) {
      if (Array.isArray(searchResult)) {
        allResults = searchResult;
        totalCount = searchResult.length;
      } else if (typeof searchResult === 'object') {
        if (Array.isArray(searchResult.results)) {
          allResults = searchResult.results;
          totalCount = searchResult.totalCount || 0;
        } else if (searchResult.totalCount !== undefined) {
          totalCount = searchResult.totalCount;
        }
      }
    }
    
    logger.debug(`${FILE_NAME}: Found ${allResults.length} results of ${totalCount} total`);
    
    // Handle no results case
    if (allResults.length === 0 && totalCount === 0) {
      return createEmptyResponse(
        `No rules found matching "${keywordValidation.trimmedKeyword}"`,
        createPaginationInfo(validatedPage, validatedPageSize, 0)
      );
    }
    
    // Create pagination info
    const pagination = createPaginationInfo(validatedPage, validatedPageSize, totalCount);
    
    // Check for valid page number
    if (offset >= totalCount && totalCount > 0) {
      return createEmptyResponse(
        `No results on page ${validatedPage}. Try a previous page.`,
        pagination
      );
    }
    
    // Return successful results
    return {
      success: true,
      results: allResults,
      count: allResults.length,
      pagination
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error searching for rules: ${error.message}`);
    return {
      success: false,
      message: `Error searching for rules: ${error.message}`
    };
  }
}

/**
 * Searches and converts Sigma rules (with full rule details)
 * @param {string} keyword - Keyword to search for
 * @param {number} page - Page number (1-based)
 * @param {number} pageSize - Results per page
 * @returns {Promise<Object>} Search results with converted rules
 */
async function searchAndConvertRules(keyword, page = 1, pageSize = 10) {
  try {
    // Perform the basic search
    const searchResult = await searchSigmaRules(keyword, page, pageSize);
    
    if (!searchResult.success || !searchResult.results || searchResult.results.length === 0) {
      return searchResult;
    }
    
    logger.debug(`${FILE_NAME}: Converting ${searchResult.results.length} search results to full rule objects`);
    
    // Convert each rule to full rule object
    const convertedResults = [];
    for (const result of searchResult.results) {
      try {
        const conversionResult = await convertSigmaRule(result.id);
        if (conversionResult.success && conversionResult.rule) {
          convertedResults.push(conversionResult.rule);
        } else {
          logger.warn(`${FILE_NAME}: Failed to convert rule ${result.id}`);
        }
      } catch (error) {
        logger.error(`${FILE_NAME}: Error converting rule ${result.id}: ${error.message}`);
      }
    }
    
    // Return results with original pagination info
    return {
      success: true,
      results: convertedResults,
      count: convertedResults.length,
      originalCount: searchResult.results.length,
      pagination: searchResult.pagination
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error in searchAndConvertRules: ${error.message}`);
    return {
      success: false,
      message: `Error searching and converting rules: ${error.message}`
    };
  }
}

module.exports = {
  searchSigmaRules,
  searchAndConvertRules
};