/**
 * search_pagination_utils.js
 * 
 * Utility functions for handling search pagination
 */
const logger = require('../../../utils/logger');
const { getFileName } = require('../../../utils/file_utils');
const FILE_NAME = getFileName(__filename);

/**
 * Creates a standard pagination info object
 * @param {number} page - Current page number (1-based)
 * @param {number} pageSize - Size of each page
 * @param {number} totalCount - Total number of results
 * @returns {Object} Standardized pagination object
 */
function createPaginationInfo(page, pageSize, totalCount) {
  const totalPages = Math.ceil(totalCount / pageSize);
  const offset = (page - 1) * pageSize;
  const hasMore = (offset + pageSize) < totalCount;
  
  return {
    currentPage: page,
    pageSize: pageSize,
    totalPages: totalPages,
    totalResults: totalCount,
    hasMore: hasMore
  };
}

/**
 * Extracts pagination parameters from command text
 * @param {string} text - The command text to parse
 * @returns {Object} Extracted parameters and cleaned text
 */
function extractPaginationParams(text) {
  if (!text) return { text: '', page: 1, pageSize: 10 };
  
  let cleanedText = text.trim();
  let page = 1;
  let pageSize = 10;

  // Extract page parameter
  const pagingMatch = cleanedText.match(/(.+)\s+page=(\d+)$/i);
  if (pagingMatch) {
    cleanedText = pagingMatch[1].trim();
    page = parseInt(pagingMatch[2], 10) || 1;
    logger.debug(`${FILE_NAME}: Extracted page ${page} from command text`);
  }

  // Extract limit parameter
  const limitMatch = cleanedText.match(/(.+)\s+limit=(\d+)$/i);
  if (limitMatch) {
    cleanedText = limitMatch[1].trim();
    pageSize = parseInt(limitMatch[2], 10) || 10;
    logger.debug(`${FILE_NAME}: Extracted limit ${pageSize} from command text`);
  }

  return {
    text: cleanedText,
    page,
    pageSize
  };
}

/**
 * Creates a standard response object for empty result sets
 * @param {string} message - Message to include in the response
 * @param {Object} pagination - Pagination info
 * @returns {Object} Standardized empty response
 */
function createEmptyResponse(message, pagination) {
  return {
    success: true,
    results: [],
    message,
    pagination
  };
}

module.exports = {
  createPaginationInfo,
  extractPaginationParams,
  createEmptyResponse
};